""" Data dictionary for Boston housing dataset. """


def boston_housing_data_dictionary() -> str:
    """Display data dictionary for Boston housing dataset."""
    return """
        **CRIM**: Per capita crime rate by town

        **ZN**: Proportion of residential land zoned for lots over 25,000 sq. ft

        **INDUS**: Proportion of non-retail business acres per town

        **CHAS**: Charles River dummy variable (= 1 if tract bounds river; 0 otherwise)

        **NOX**: Nitric oxide concentration (parts per 10 million)

        **RM**: Average number of rooms per dwelling

        **AGE**: Proportion of owner-occupied units built prior to 1940

        **DIS**: Weighted distances to five Boston employment centers

        **RAD**: Index of accessibility to radial highways

        **TAX**: Full-value property tax rate per $10,000

        **PTRATIO**: Pupil-teacher ratio by town

        **B**: $1000(Bk — 0.63)²$, where Bk is the proportion of
            [people of African American descent] by town

        **LSTAT**: Percentage of lower status of the population

        **MEDV**: Median value of owner-occupied homes in $1000s
        """
